/* This software is modified from
 * https://github.com/sparkfun/MMA8452_Accelerometer
 *  MMA8452Q Basic Example Code
 *  Nathan Seidle
 *  SparkFun Electronics
 *  November 5, 2012
 *
 * This code is public domain.
*/
/*
 *   Modified 18 March 2014 by yuuki.okamiya.yn@renesas.com
 *   Modified for KX022 by masashi.fujimoto.eb@renesas.com on 3rd September 2015
 */
#include <Arduino.h>
#include "KX022.h"
#include <wire.h>

void readAccelData(float *destination)
{
  byte rawData[6];  // x/y/z accel register data stored here
  readRegisters(XOUT_L, 6, rawData);  // Read the six raw data registers into data array
  int temp_dest[3];
  // Loop to calculate 16-bit ADC and g value for each axis
  for(int i = 0; i < 3 ; i++)
  {
	int gCount;
    if(HIGHCURRENT==1){
      gCount = (rawData[(i*2)+1] << 8) | rawData[i*2];  //Combine the two 8 bit registers into one 12-bit number

      // If the number is negative, we have to make it so manually (no 12-bit data type)
      if (rawData[(i*2)+1] > 0x7F)
      {
        gCount = ~gCount + 1;
        gCount *= -1;  // Transform into negative 2's complement #
      }
    }
    else {
      gCount = rawData[i*2];  //Combine the two 8 bit registers into one 12-bit number

      // If the number is negative, we have to make it so manually (no 12-bit data type)
      if (rawData[i*2] > 0x7F)
      {
        gCount = ~gCount + 1;
        gCount *= -1;  // Transform into negative 2's complement #
      }
    }
    temp_dest[i] = gCount; //Record this gCount into the 3 int array
  }

  // Now we'll calculate the accleration value into actual g's
  // Stores the real accel value in g's
  for (int i = 0 ; i < 3 ; i++)
  {
    if(HIGHCURRENT==1){
      destination[i] = (float) temp_dest[i] / (0x8000/GSCALE);  // get actual g value, 16bit ADC
    }
    else {
      destination[i] = (float) temp_dest[i] / (0x80/GSCALE);  // get actual g value, 8bit ADC
    }
  }
}

// Initialize the MMA8452 registers
// See the many application notes for more info on setting all of these registers:
// http://www.freescale.com/webapp/sps/site/prod_summary.jsp?code=MMA8452Q
bool initKX022()
{
  Wire.begin(); //Join the bus as a master

  byte c = readRegister(WHO_AM_I);  // Read WHO_AM_I register
  if (c != 0x14) // WHO_AM_I should always be 0x2A
  {
//      return 0;
  }

  KX022Standby();  // Must be in standby to change registers

  byte fsr;
  // Set up the full scale range to 2, 4, or 8g.
  switch(GSCALE){
    case 2 : fsr = 0x00; break;
    case 4 : fsr = 0x04; break;
    case 8 : fsr = 0x08; break;
    default: fsr = 0x00; break;
  }
  if(HIGHCURRENT==1) fsr = fsr | 0x40;
  else               fsr = fsr & 0x40;
  
  writeRegister(CNTL1, fsr);

  //The default data rate is 800Hz and we don't modify it in this example code

  KX022Active();  // Set to active to start reading
  return 1;
}

// Sets the KX022 to standby mode. It must be in standby to change most register settings
void KX022Standby()
{
  byte c = readRegister(CNTL1);
  writeRegister(CNTL1, c & ~(0x80)); //Clear the active bit(CNTL1.PC1) to go into standby
}

// Sets the KX022 to active mode. Needs to be in this mode to output data
void KX022Active()
{
  byte c = readRegister(CNTL1);
  writeRegister(CNTL1, c | 0x80); //Set the active bit to begin detection
}

// Read bytesToRead sequentially, starting at addressToRead into the dest byte array
void readRegisters(byte addressToRead, int bytesToRead, byte * dest)
{
  Wire.beginTransmission(KX022_ADDRESS);
  Wire.write(addressToRead);
  Wire.endTransmission(false); //endTransmission but keep the connection active

  Wire.requestFrom(KX022_ADDRESS, bytesToRead); //Ask for bytes, once done, bus is released by default

  while(Wire.available() < bytesToRead); //Hang out until we get the # of bytes we expect

  for(int x = 0 ; x < bytesToRead ; x++)
    dest[x] = Wire.read();
}

// Read a single byte from addressToRead and return it as a byte
byte readRegister(byte addressToRead)
{
  Wire.beginTransmission(KX022_ADDRESS);
  Wire.write(addressToRead);
  Wire.endTransmission(false); //endTransmission but keep the connection active

  Wire.requestFrom(KX022_ADDRESS, 1); //Ask for 1 byte, once done, bus is released by default

  while(!Wire.available()) ; //Wait for the data to come back
  return Wire.read(); //Return this one byte
}

// Writes a single byte (dataToWrite) into addressToWrite
void writeRegister(byte addressToWrite, byte dataToWrite)
{
  Wire.beginTransmission(KX022_ADDRESS);
  Wire.write(addressToWrite);
  Wire.write(dataToWrite);
  Wire.endTransmission(); //Stop transmitting
}
